@@ -5,18 +5,20 @@
 
 #include "libnetdata/libnetdata.h"
 
-#define ACLK_STABLE_TIMEOUT 3 // Minimum delay to mark AGENT as stable
+#include "mqtt_wss_client.h"
 
 extern pthread_cond_t query_cond_wait;
 extern pthread_mutex_t query_lock_wait;
 #define QUERY_THREAD_WAKEUP pthread_cond_signal(&query_cond_wait)
 #define QUERY_THREAD_WAKEUP_ALL pthread_cond_broadcast(&query_cond_wait)
 
-extern volatile int aclk_connected;
+// TODO
+//extern volatile int aclk_connected;
 
 struct aclk_query_thread {
     netdata_thread_t thread;
     int idx;
+    mqtt_wss_client client;
 };
 
 struct aclk_query_threads {
@@ -24,11 +26,7 @@ struct aclk_query_threads {
     int count;
 };
 
-void *aclk_query_main_thread(void *ptr);
-int aclk_queue_query(char *token, char *data, char *msg_type, char *query, int run_after, int internal, ACLK_CMD cmd);
-
-void aclk_query_threads_start(struct aclk_query_threads *query_threads);
+void aclk_query_threads_start(struct aclk_query_threads *query_threads, mqtt_wss_client client);
 void aclk_query_threads_cleanup(struct aclk_query_threads *query_threads);
-unsigned int aclk_query_size();
 
 #endif //NETDATA_AGENT_CLOUD_LINK_H
