@@ -152,16 +152,34 @@ PARSER_RC pluginsd_label_action(void *user, char *key, char *value, LABEL_SOURCE
     return PARSER_RC_OK;
 }
 
+PARSER_RC pluginsd_clabel_action(void *user, char *key, char *value, LABEL_SOURCE source)
+{
+    ((PARSER_USER_OBJECT *) user)->chart_labels = add_label_to_list(((PARSER_USER_OBJECT *) user)->chart_labels, key, value, source);
+
+    return PARSER_RC_OK;
+}
+
+PARSER_RC pluginsd_clabel_commit_action(void *user, RRDHOST *host, struct label *new_labels)
+{
+    RRDSET *st = ((PARSER_USER_OBJECT *)user)->st;
+    if (unlikely(!st)) {
+        error("requested CLABEL_COMMIT on host '%s', without a BEGIN, ignoring it.", host->hostname);
+        return PARSER_RC_OK;
+    }
+
+    rrdset_update_labels(st, new_labels);
+    return PARSER_RC_OK;
+}
 
 PARSER_RC pluginsd_overwrite_action(void *user, RRDHOST *host, struct label *new_labels)
 {
     UNUSED(user);
 
-    if (!host->labels) {
-        host->labels = new_labels;
+    if (!host->labels.head) {
+        host->labels.head = new_labels;
     } else {
         rrdhost_rdlock(host);
-        replace_label_list(host, new_labels);
+        replace_label_list(&host->labels, new_labels);
         rrdhost_unlock(host);
     }
     return PARSER_RC_OK;
@@ -274,6 +292,10 @@ PARSER_RC pluginsd_end(char **words, void *user, PLUGINSD_ACTION  *plugins_actio
 PARSER_RC pluginsd_chart(char **words, void *user, PLUGINSD_ACTION  *plugins_action)
 {
     RRDHOST *host = ((PARSER_USER_OBJECT *) user)->host;
+    if (unlikely(!host && !((PARSER_USER_OBJECT *) user)->host_exists)) {
+        debug(D_PLUGINSD, "Ignoring chart belonging to missing or ignored host.");
+        return PARSER_RC_OK;
+    }
 
     char *type = words[1];
     char *name = words[2];
@@ -299,7 +321,10 @@ PARSER_RC pluginsd_chart(char **words, void *user, PLUGINSD_ACTION  *plugins_act
 
     // make sure we have the required variables
     if (unlikely((!type || !*type || !id || !*id))) {
-        error("requested a CHART, without a type.id, on host '%s'. Disabling it.", host->hostname);
+        if (likely(host))
+            error("requested a CHART, without a type.id, on host '%s'. Disabling it.", host->hostname);
+        else
+            error("requested a CHART, without a type.id. Disabling it.");
         ((PARSER_USER_OBJECT *) user)->enabled = 0;
         return PARSER_RC_ERROR;
     }
@@ -371,6 +396,10 @@ PARSER_RC pluginsd_dimension(char **words, void *user, PLUGINSD_ACTION  *plugins
 
     RRDSET *st = ((PARSER_USER_OBJECT *) user)->st;
     RRDHOST *host = ((PARSER_USER_OBJECT *) user)->host;
+    if (unlikely(!host && !((PARSER_USER_OBJECT *) user)->host_exists)) {
+        debug(D_PLUGINSD, "Ignoring dimension belonging to missing or ignored host.");
+        return PARSER_RC_OK;
+    }
 
     if (unlikely(!id)) {
         error(
@@ -379,7 +408,7 @@ PARSER_RC pluginsd_dimension(char **words, void *user, PLUGINSD_ACTION  *plugins
         goto disable;
     }
 
-    if (unlikely(!st)) {
+    if (unlikely(!st && !((PARSER_USER_OBJECT *) user)->st_exists)) {
         error("requested a DIMENSION, without a CHART, on host '%s'. Disabling it.", host->hostname);
         goto disable;
     }
@@ -401,7 +430,7 @@ PARSER_RC pluginsd_dimension(char **words, void *user, PLUGINSD_ACTION  *plugins
     if (unlikely(!algorithm || !*algorithm))
         algorithm = "absolute";
 
-    if (unlikely(rrdset_flag_check(st, RRDSET_FLAG_DEBUG)))
+    if (unlikely(st && rrdset_flag_check(st, RRDSET_FLAG_DEBUG)))
         debug(
             D_PLUGINSD,
             "creating dimension in chart %s, id='%s', name='%s', algorithm='%s', multiplier=%ld, divisor=%ld, hidden='%s'",
@@ -509,6 +538,11 @@ PARSER_RC pluginsd_disable(char **words, void *user, PLUGINSD_ACTION  *plugins_a
 PARSER_RC pluginsd_label(char **words, void *user, PLUGINSD_ACTION  *plugins_action)
 {
     char *store;
+
+    if (!words[1] || !words[2] || !words[3]) {
+        error("Ignoring malformed or empty LABEL command.");
+        return PARSER_RC_OK;
+    }
     if (!words[4])
         store = words[3];
     else {
@@ -544,12 +578,44 @@ PARSER_RC pluginsd_label(char **words, void *user, PLUGINSD_ACTION  *plugins_act
     return PARSER_RC_OK;
 }
 
+PARSER_RC pluginsd_clabel(char **words, void *user, PLUGINSD_ACTION  *plugins_action)
+{
+    if (!words[1] || !words[2] || !words[3]) {
+        error("Ignoring malformed or empty CHART LABEL command.");
+        return PARSER_RC_OK;
+    }
+
+    if (plugins_action->clabel_action) {
+        PARSER_RC rc = plugins_action->clabel_action(user, words[1], words[2], strtol(words[3], NULL, 10));
+        return rc;
+    }
+
+    return PARSER_RC_OK;
+}
+
+PARSER_RC pluginsd_clabel_commit(char **words, void *user, PLUGINSD_ACTION  *plugins_action)
+{
+    UNUSED(words);
+
+    RRDHOST *host = ((PARSER_USER_OBJECT *) user)->host;
+    debug(D_PLUGINSD, "requested to commit chart labels");
+
+    struct label *chart_labels = ((PARSER_USER_OBJECT *)user)->chart_labels;
+    ((PARSER_USER_OBJECT *)user)->chart_labels = NULL;
+
+    if (plugins_action->clabel_commit_action) {
+        return plugins_action->clabel_commit_action(user, host, chart_labels);
+    }
+
+    return PARSER_RC_OK;
+}
+
 PARSER_RC pluginsd_overwrite(char **words, void *user, PLUGINSD_ACTION  *plugins_action)
 {
     UNUSED(words);
 
     RRDHOST *host = ((PARSER_USER_OBJECT *) user)->host;
-    debug(D_PLUGINSD, "requested a OVERWITE a variable");
+    debug(D_PLUGINSD, "requested a OVERWRITE a variable");
 
     struct label *new_labels = ((PARSER_USER_OBJECT *)user)->new_labels;
     ((PARSER_USER_OBJECT *)user)->new_labels = NULL;
@@ -627,6 +693,33 @@ PARSER_RC pluginsd_tombstone(char **words, void *user, PLUGINSD_ACTION *plugins_
     return PARSER_RC_OK;
 }
 
+PARSER_RC metalog_pluginsd_host(char **words, void *user, PLUGINSD_ACTION  *plugins_action)
+{
+    char *machine_guid = words[1];
+    char *hostname = words[2];
+    char *registry_hostname = words[3];
+    char *update_every_s = words[4];
+    char *os = words[5];
+    char *timezone = words[6];
+    char *tags = words[7];
+
+    int update_every = 1;
+    if (likely(update_every_s && *update_every_s))
+        update_every = str2i(update_every_s);
+    if (unlikely(!update_every))
+        update_every = 1;
+
+    debug(D_PLUGINSD, "HOST PARSED: guid=%s, hostname=%s, reg_host=%s, update=%d, os=%s, timezone=%s, tags=%s",
+         machine_guid, hostname, registry_hostname, update_every, os, timezone, tags);
+
+    if (plugins_action->host_action) {
+        return plugins_action->host_action(
+            user, machine_guid, hostname, registry_hostname, update_every, os, timezone, tags);
+    }
+
+    return PARSER_RC_OK;
+}
+
 // New plugins.d parser
 
 inline size_t pluginsd_process(RRDHOST *host, struct plugind *cd, FILE *fp, int trust_durations)
